/*
Transformée de Fourier

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU.

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/


new TF();

// constructeur de la classe, avec la définition des paramètres de simulation
// et des options d'affichage.
function TF() {
    this.applet = new SimuApplet(this);
    this.applet.appletDoc("tf_doc.xml");
}

TF.prototype.calcul = function(lin) {
    var fonction, duree, periode, periode2, methode, amplitude, phase, bruit, npts, tfx, tfy, tspecx, tspecy, out, i, info, dt, fc;
    
    fonction = lin.lireString("fonction");
    duree = lin.lireDouble("duree");
    periode = lin.lireDouble("periode");
    periode2 = lin.lireDouble("periode2");
    methode = lin.lireString("methode");
    amplitude = lin.lireDouble("amplitude");
    phase = lin.lireDouble("phase");
    bruit = lin.lireDouble("bruit");
    if (periode <= 0)
        periode = 0.001;
    npts = lin.lireEntier("npts");
    if (npts < 1)
        npts = 1;
    
    tfx = this.calcul_fx(npts, 0., duree);
    tfy = this.calcul_fy(fonction, periode, periode2, tfx, amplitude, phase, bruit);
    tspecx = this.calcul_specx(tfx, duree);
    tspecy = this.calcul_specy(tfx, tfy, tspecx, methode);
    
    out = new ListeValeurs();
    for (i=0; i<npts; i++) {
        out.ajouterDouble("fx", tfx[i]);
        out.ajouterDouble("fy", tfy[i]);
    }
    for (i=0; i<tspecx.length; i++) {
        out.ajouterDouble("specx", tspecx[i]);
        out.ajouterDouble("specy", tspecy[i]);
    }
    
    info = "résolution: " + this.joliFormattage(1 / duree, 3) + " Hz";
    dt = duree / (npts - 1);
    fc = 1 / (2*dt);
    info += "  fréquence de coupure: " + this.joliFormattage(fc, 3) + " Hz";
    out.ajouterString("info", info);
    
    return out;
}

TF.prototype.joliFormattage = function(d, precision) {
    if (isNaN(d))
        return('');
    if (d == 0)
        return('0');
    var facteur_precision = Math.pow(10, precision);
    var exposant = Math.floor(Math.log(Math.abs(d))/Math.log(10));
    var mantisse = d / (Math.pow(10,exposant));
    var s;
    if (exposant == 0) {
        mantisse = Math.round(mantisse*facteur_precision)/facteur_precision;
        s = ""+mantisse;
    } else if (exposant == 1) {
        mantisse = Math.round(mantisse*facteur_precision)/(facteur_precision/10);
        s = ""+mantisse;
    } else if (exposant == -1) {
        var signe;
        if (mantisse < 0)
            signe = "-";
        else
            signe = "";
        mantisse = Math.abs(Math.round(mantisse*facteur_precision));
        s = signe + "0."+Math.floor(mantisse);
        s = s.replace(/0+$/g, '');
    } else {
        mantisse = Math.round(mantisse*facteur_precision)/facteur_precision;
        s = mantisse + "E" + exposant;
    }
    return(s);
}

//public double[] calcul_fx(int npts, double x0, double x1) {
TF.prototype.calcul_fx = function(npts, x0, x1) {
    var tfx, i;
    tfx = new Array(npts);
    for (i=0; i<npts; i++)
        tfx[i] = x0 + i*(x1 - x0)/(npts - 1);
    return(tfx);
}

//public double[] calcul_fy(String fonction, double periode, double periode2, double[] tfx, double amplitude,
//        double phase, double bruit) {
TF.prototype.calcul_fy = function(fonction, periode, periode2, tfx, amplitude, phase, bruit) {
    var tfy;
    
    if (fonction == "porte")
        tfy = this.fonction_porte(tfx, amplitude, phase, bruit);
    else if (fonction == "créneau")
        tfy = this.fonction_creneau(tfx, periode, amplitude, phase, bruit);
    else if (fonction == "sinusoïde")
        tfy = this.fonction_sinusoide(tfx, periode, amplitude, phase, bruit);
    else if (fonction == "2 sinusoïdes")
        tfy = this.fonction_double_sinusoide(tfx, periode, periode2, amplitude, phase, bruit);
    else
        tfy = null;
    
    return(tfy);
}

//public double[] calcul_specx(double[] tfx, double duree) {
TF.prototype.calcul_specx = function(tfx, duree) {
    var tspecx, dn, i;
    tspecx = new Array(Math.floor(tfx.length/2));
    dn = 1/duree;
    for (i=0; i<tspecx.length; i++)
        tspecx[i] = i*dn;
    return(tspecx);
}

//public double[] calcul_specy(double[] tfx, double[] tfy, double[] tspecx, String methode) {
TF.prototype.calcul_specy = function(tfx, tfy, tspecx, methode) {
    var tspecy, n, f_r, f_i, i, temp, temp_cos, temp_sin, k;
    tspecy = new Array(tspecx.length);
    if (methode == "FFT") {
        // arrondi supérieur pour avoir une puissance de 2
        n = Math.floor(Math.pow(2, Math.ceil(Math.log(tfx.length)/Math.log(2))));
        if (n != tfx.length)
            alert("FFT: " + tfx.length + " arrondi a " + n + ", remplissage avec des zeros");
        f_r = new Array(n);
        f_i = new Array(n);
        for (i=0; i<n; i++) {
            if (i < tfy.length)
                f_r[i] = tfy[i];
            else
                f_r[i] = 0;
            f_i[i] = 0;
        }
        TFLib.fft_rec(false, f_r, f_i);
        for (i=0; i<tspecx.length; i++)
            tspecy[i] = f_r[i]*f_r[i] + f_i[i]*f_i[i];
    } else {
        for (i=0; i<tspecx.length; i++) {
            temp_cos = 0;
            temp_sin = 0;
            for (k=0; k<tfx.length; k++) {
                temp = 2 * Math.PI * tfx[k] * tspecx[i];
                temp_cos += tfy[k] * Math.cos(temp);
                temp_sin += tfy[k] * Math.sin(temp);
            }
            tspecy[i] = temp_cos*temp_cos + temp_sin*temp_sin;
        }
    }
    return(tspecy);
}

    
//public static double[] fonction_porte(double[] tfx, double amplitude, double phase, double bruit) {
TF.prototype.fonction_porte = function(tfx, amplitude, phase, bruit) {
    var tfy, i;
    
    tfy = new Array(tfx.length);
    for (i=0; i<tfx.length; i++) {
        if (tfx[i] < phase || tfx[i] >= phase + tfx[tfx.length - 1]/2)
            tfy[i] = 0;
        else
            tfy[i] = amplitude;
        tfy[i] += bruit * Math.random();
    }
    return(tfy);
}

//public static double[] fonction_creneau(double[] tfx, double periode, double amplitude, double phase, double bruit) {
TF.prototype.fonction_creneau = function(tfx, periode, amplitude, phase, bruit) {
    var tfy, i, xmod;
    
    tfy = new Array(tfx.length);
    for (i=0; i<tfx.length; i++) {
        xmod = (tfx[i] + phase) / periode;
        xmod = (xmod - Math.floor(xmod)) * periode;
        if (xmod < periode/2)
            tfy[i] = 0;
        else
            tfy[i] = amplitude;
        tfy[i] += bruit * Math.random();
    }
    return(tfy);
}

//public static double[] fonction_sinusoide(double[] tfx, double periode, double amplitude, double phase, double bruit) {
TF.prototype.fonction_sinusoide = function(tfx, periode, amplitude, phase, bruit) {
    var tfy, i;
    
    tfy = new Array(tfx.length);
    for (i=0; i<tfx.length; i++)
        tfy[i] = amplitude*Math.sin( (tfx[i] + phase) * 2 * Math.PI / periode ) + bruit * Math.random();
    return(tfy);
}

//public static double[] fonction_double_sinusoide(double[] tfx, double periode, double periode2, double amplitude,
//        double phase, double bruit) {
TF.prototype.fonction_double_sinusoide = function(tfx, periode, periode2, amplitude, phase, bruit) {
    var tfy, i;
    
    tfy = new Array(tfx.length);
    for (i=0; i<tfx.length; i++)
        tfy[i] = Math.sin( tfx[i] * 2 * Math.PI / periode ) +
            amplitude*Math.sin( (tfx[i] + phase) * 2 * Math.PI / periode2 ) +
                bruit * Math.random();
    return(tfy);
}
